@@ -293,6 +293,77 @@ signal_name_to_dbus (const gchar *s)
   return ret;
 }
 
+/*
+ * Converts names of the form "active-descendant-changed" to
+ * "ActiveDescendantChanged"
+ */
+static gchar *
+ensure_proper_format (const char *name)
+{
+  gchar *ret = (gchar *) g_malloc (strlen (name) * 2 + 2);
+  gchar *p = ret;
+  gboolean need_upper = TRUE;
+
+  if (!ret)
+    return NULL;
+  while (*name)
+    {
+      if (need_upper)
+        {
+          *p++ = toupper (*name);
+          need_upper = FALSE;
+        }
+      else if (*name == '-')
+        need_upper = TRUE;
+      else if (*name == ':')
+        {
+          need_upper = TRUE;
+          *p++ = *name;
+        }
+      else
+        *p++ = *name;
+      name++;
+    }
+  *p = '\0';
+  return ret;
+}
+
+static gboolean
+signal_is_needed (const gchar *klass, const gchar *major, const gchar *minor)
+{
+  gchar *data [4];
+  GList *iter;
+  event_data *evdata;
+  gboolean ret = FALSE;
+  GList *list;
+
+  if (!spi_global_app_data->events_initialized)
+    return TRUE;
+
+  data [0] = ensure_proper_format (klass + 21);
+  data [1] = ensure_proper_format (major);
+  data [2] = ensure_proper_format (minor);
+  data [3] = NULL;
+
+  /* Hack: events such as "object::text-changed::insert:system" as
+     generated by Gecko */
+  data [2][strcspn (data [2], ":")] = '\0';
+  for (list = spi_global_app_data->events; list; list = list->next)
+    {
+      evdata = list->data;
+      if (spi_event_is_subtype (data, evdata->data))
+        {
+          ret = TRUE;
+          break;
+        }
+    }
+
+  g_free (data [2]);
+  g_free (data [1]);
+  g_free (data [0]);
+  return ret;
+}
+
 /*
  * Emits an AT-SPI event.
  * AT-SPI events names are split into three parts:
@@ -327,6 +398,9 @@ emit_event (AtkObject  *obj,
   if (!minor) minor = "";
   if (!type) type = "u";
 
+  if (!signal_is_needed (klass, major, minor))
+    return;
+
   /*
    * This is very annoying, but as '-' isn't a legal signal
    * name in D-Bus (Why not??!?) The names need converting
@@ -1022,4 +1096,17 @@ spi_atk_tidy_windows (void)
     }
 }
 
+gboolean
+spi_event_is_subtype (gchar **needle, gchar **haystack)
+{
+  while (*haystack && **haystack)
+    {
+      if (g_strcmp0 (*needle, *haystack))
+        return FALSE;
+      needle++;
+      haystack++;
+    }
+  return TRUE;
+}
+
 /*END------------------------------------------------------------------------*/
